import abc
from collections.abc import Callable
from datetime import datetime, timedelta, tzinfo
from typing import ClassVar, TypeVar
from typing_extensions import ParamSpec

ZERO: timedelta

__all__ = ["tzname_in_python2", "enfold"]

_P = ParamSpec("_P")
_R = TypeVar("_R")
_DateTimeT = TypeVar("_DateTimeT", bound=datetime)

def tzname_in_python2(namefunc: Callable[_P, _R]) -> Callable[_P, _R]: ...
def enfold(dt: _DateTimeT, fold: int = 1) -> _DateTimeT: ...

# Doesn't actually have ABCMeta as the metaclass at runtime,
# but mypy complains if we don't have it in the stub.
# See discussion in #8908
class _tzinfo(tzinfo, metaclass=abc.ABCMeta):
    def is_ambiguous(self, dt: datetime) -> bool: ...
    def fromutc(self, dt: datetime) -> datetime: ...

class tzrangebase(_tzinfo):
    def __init__(self) -> None: ...
    def utcoffset(self, dt: datetime | None) -> timedelta | None: ...
    def dst(self, dt: datetime | None) -> timedelta | None: ...
    def tzname(self, dt: datetime | None) -> str: ...
    def fromutc(self, dt: datetime) -> datetime: ...
    def is_ambiguous(self, dt: datetime) -> bool: ...
    __hash__: ClassVar[None]  # type: ignore[assignment]
    def __ne__(self, other: object) -> bool: ...
    __reduce__ = object.__reduce__



from datetime import _TzInfo, datetime, timedelta
from typing import TypeVar

_DateTimeT = TypeVar("_DateTimeT", bound=datetime)

def today(tzinfo: _TzInfo | None = None) -> datetime: ...
def default_tzinfo(dt: _DateTimeT, tzinfo: _TzInfo) -> _DateTimeT: ...
def within_delta(dt1: datetime, dt2: datetime, delta: timedelta) -> bool: ...



from ._parser import (
    DEFAULTPARSER as DEFAULTPARSER,
    DEFAULTTZPARSER as DEFAULTTZPARSER,
    ParserError as ParserError,
    UnknownTimezoneWarning as UnknownTimezoneWarning,
    parse as parse,
    parser as parser,
    parserinfo as parserinfo,
)
from .isoparser import isoparse as isoparse, isoparser as isoparser

__all__ = ["parse", "parser", "parserinfo", "isoparse", "isoparser", "ParserError", "UnknownTimezoneWarning"]



import sys
from ctypes import _NameTypes, _Pointer, c_wchar
from datetime import datetime, timedelta
from typing import Any, ClassVar, Final

from dateutil.tz import tzwin as tzwin, tzwinlocal as tzwinlocal

if sys.platform == "win32":
    from winreg import _KeyType

    __all__ = ["tzwin", "tzwinlocal", "tzres"]

    ONEWEEK: timedelta
    TZKEYNAMENT: Final[str]
    TZKEYNAME9X: Final[str]
    TZLOCALKEYNAME: Final[str]
    TZKEYNAME: Final[str]

    class tzres:
        p_wchar: ClassVar[type[_Pointer[c_wchar]]]
        tzres_loc: _NameTypes
        def __init__(self, tzres_loc: _NameTypes = "tzres.dll") -> None: ...
        def load_name(self, offset: int) -> str: ...
        def name_from_string(self, tzname_str: str) -> str: ...

    def picknthweekday(year: int, month: int, dayofweek: int, hour: int, minute: int, whichweek: int) -> datetime: ...
    def valuestodict(key: _KeyType) -> dict[str, Any]: ...  # keys and values in dict are results of winreg.EnumValue() function



import re
from _typeshed import SupportsRead
from collections.abc import Callable, Mapping
from datetime import _TzInfo, datetime
from io import StringIO
from typing import IO, Any, Literal, overload
from typing_extensions import Self, TypeAlias

_FileOrStr: TypeAlias = bytes | str | IO[str] | IO[Any]
_TzData: TypeAlias = _TzInfo | int | str | None
_TzInfos: TypeAlias = Mapping[str, _TzData] | Callable[[str, int], _TzData]

__all__ = ["parse", "parserinfo", "ParserError"]

class _timelex:
    _split_decimal: re.Pattern[str]
    instream: StringIO | SupportsRead[str]
    charstack: list[str]
    tokenstack: list[str]
    eof: bool
    def __init__(self, instream: str | bytes | bytearray | SupportsRead[str]) -> None: ...
    def get_token(self) -> str | None: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> str: ...
    def next(self) -> str: ...
    @classmethod
    def split(cls, s: str) -> list[str]: ...
    @classmethod
    def isword(cls, nextchar: str) -> bool: ...
    @classmethod
    def isnum(cls, nextchar: str) -> bool: ...
    @classmethod
    def isspace(cls, nextchar: str) -> bool: ...

class _resultbase:
    def __init__(self) -> None: ...
    def _repr(self, classname: str) -> str: ...
    def __len__(self) -> int: ...

class parserinfo:
    JUMP: list[str]
    WEEKDAYS: list[tuple[str, str]]
    MONTHS: list[tuple[str, str] | tuple[str, str, str]]
    HMS: list[tuple[str, str, str]]
    AMPM: list[tuple[str, str]]
    UTCZONE: list[str]
    PERTAIN: list[str]
    TZOFFSET: dict[str, int]
    def __init__(self, dayfirst: bool = False, yearfirst: bool = False) -> None: ...
    def jump(self, name: str) -> bool: ...
    def weekday(self, name: str) -> int | None: ...
    def month(self, name: str) -> int | None: ...
    def hms(self, name: str) -> int | None: ...
    def ampm(self, name: str) -> int | None: ...
    def pertain(self, name: str) -> bool: ...
    def utczone(self, name: str) -> bool: ...
    def tzoffset(self, name: str) -> int | None: ...
    def convertyear(self, year: int, century_specified: bool = False) -> int: ...
    def validate(self, res: datetime) -> bool: ...

class _ymd(list[int]):
    century_specified: bool
    dstridx: int | None
    mstridx: int | None
    ystridx: int | None
    @property
    def has_year(self) -> bool: ...
    @property
    def has_month(self) -> bool: ...
    @property
    def has_day(self) -> bool: ...
    def could_be_day(self, value: int) -> bool: ...
    def append(self, val: str | int, label: str | None = None) -> None: ...
    def _resolve_from_stridxs(self, strids: dict[str, int]) -> tuple[int, int, int]: ...
    def resolve_ymd(self, yearfirst: bool | None, dayfirst: bool | None) -> tuple[int, int, int]: ...

class parser:
    info: parserinfo
    def __init__(self, info: parserinfo | None = None) -> None: ...
    @overload
    def parse(
        self,
        timestr: _FileOrStr,
        default: datetime | None = None,
        ignoretz: bool = False,
        tzinfos: _TzInfos | None = None,
        *,
        dayfirst: bool | None = ...,
        yearfirst: bool | None = ...,
        fuzzy: bool = ...,
        fuzzy_with_tokens: Literal[False] = False,
    ) -> datetime: ...
    @overload
    def parse(
        self,
        timestr: _FileOrStr,
        default: datetime | None = None,
        ignoretz: bool = False,
        tzinfos: _TzInfos | None = None,
        *,
        dayfirst: bool | None = ...,
        yearfirst: bool | None = ...,
        fuzzy: bool = ...,
        fuzzy_with_tokens: Literal[True],
    ) -> tuple[datetime, tuple[str, ...]]: ...

DEFAULTPARSER: parser

@overload
def parse(
    timestr: _FileOrStr,
    parserinfo: parserinfo | None = None,
    *,
    dayfirst: bool | None = ...,
    yearfirst: bool | None = ...,
    ignoretz: bool = ...,
    fuzzy: bool = ...,
    fuzzy_with_tokens: Literal[False] = False,
    default: datetime | None = ...,
    tzinfos: _TzInfos | None = ...,
) -> datetime: ...
@overload
def parse(
    timestr: _FileOrStr,
    parserinfo: parserinfo | None = None,
    *,
    dayfirst: bool | None = ...,
    yearfirst: bool | None = ...,
    ignoretz: bool = ...,
    fuzzy: bool = ...,
    fuzzy_with_tokens: Literal[True],
    default: datetime | None = ...,
    tzinfos: _TzInfos | None = ...,
) -> tuple[datetime, tuple[str, ...]]: ...

class _tzparser:
    class _result(_resultbase):
        __slots__ = ["stdabbr", "stdoffset", "dstabbr", "dstoffset", "start", "end"]
        stdabbr: str | None
        stdoffset: int | None
        dstabbr: str | None
        dstoffset: int | None
        start: _attr
        end: _attr

        class _attr(_resultbase):
            __slots__ = ["month", "week", "weekday", "yday", "jyday", "day", "time"]
            month: int | None
            week: int | None
            weekday: int | None
            yday: int | None
            jyday: int | None
            day: int | None
            time: int | None

        def __init__(self) -> None: ...

    def parse(self, tzstr: str | re.Pattern[str]) -> _result | None: ...

DEFAULTTZPARSER: _tzparser

class ParserError(ValueError): ...
class UnknownTimezoneWarning(RuntimeWarning): ...
